@@ -30,6 +30,10 @@
 #include "application.h"
 #include "operation.h"
 #include "theme.h"
+#include "tool.h"
+#ifndef GOFFICE_HAS_GLOBAL_HEADER
+#   include <goffice/math/go-math.h>
+#endif
 #include <cstring>
 
 using namespace gcu;
@@ -43,14 +47,15 @@ guint ClipboardDataType, ClipboardDataType1;
 bool cleared = true;
 
 GtkTargetEntry const export_targets[] = {
-	{(char *) GCHEMPAINT_ATOM_NAME,  0, 0},
-	{(char *) "image/svg",  0, 1},
-	{(char *) "image/svg+xml",  0, 2},
-	{(char *) "image/png",  0, 3},
-	{(char *) "image/jpeg",  0, 4},
-	{(char *) "image/bmp",  0, 5},
-	{(char *) "UTF8_STRING", 0, 6},
-	{(char *) "STRING", 0, 7}
+	{(char *) GCHEMPAINT_ATOM_NAME,  0, GCP_CLIPBOARD_NATIVE},
+	{(char *) "image/svg",  0, GCP_CLIPBOARD_SVG},
+	{(char *) "image/svg+xml",  0, GCP_CLIPBOARD_SVG_XML},
+	{(char *) "image/x-eps",  0, GCP_CLIPBOARD_EPS},
+	{(char *) "image/png",  0, GCP_CLIPBOARD_PNG},
+	{(char *) "image/jpeg",  0, GCP_CLIPBOARD_JPEG},
+	{(char *) "image/bmp",  0, GCP_CLIPBOARD_BMP},
+	{(char *) "UTF8_STRING", 0, GCP_CLIPBOARD_UTF8_STRING},
+	{(char *) "STRING", 0, GCP_CLIPBOARD_STRING}
 };
 
 void on_receive_targets (GtkClipboard *clipboard, GtkSelectionData *selection_data, Application *App)
@@ -63,6 +68,7 @@ void on_receive_targets (GtkClipboard *clipboard, GtkSelectionData *selection_da
 			GCHEMPAINT_ATOM_NAME,
 			"image/svg",
 			"image/svg+xml",
+			"image/x-eps",
 			"image/png",
 			"image/jpeg",
 			"image/bmp",
@@ -82,7 +88,7 @@ void on_receive_targets (GtkClipboard *clipboard, GtkSelectionData *selection_da
 		}
 
 		gchar* name;
-		*DataType = 7;
+		*DataType = GCP_CLIPBOARD_ALL;
 		for ( j = 0; j < atom_count ; j++) {
 			name = gdk_atom_name (targets [j]);
 			for (i = 0; i < *DataType; i++)
@@ -95,7 +101,7 @@ void on_receive_targets (GtkClipboard *clipboard, GtkSelectionData *selection_da
 	}
 	if (clipboard == sel_clipboard && App != NULL)
 		App->ActivateWindowsActionWidget ("/MainMenu/EditMenu/Paste",
-			ClipboardDataType == 0 || ClipboardDataType == 5 || ClipboardDataType == 6);
+			ClipboardDataType == GCP_CLIPBOARD_NATIVE || ClipboardDataType == GCP_CLIPBOARD_UTF8_STRING || ClipboardDataType == GCP_CLIPBOARD_STRING);
 }
 
 static void on_get_data (GtkClipboard *clipboard, GtkSelectionData *selection_data,  guint info, Application *App)
@@ -111,12 +117,12 @@ static void on_get_data (GtkClipboard *clipboard, GtkSelectionData *selection_da
 	*DataType = info;
 	int size;
 	switch (info) {
-	case 0:
+	case GCP_CLIPBOARD_NATIVE:
 		xmlDocDumpFormatMemory (pDoc, &ClipboardData, &size, info);
 		gtk_selection_data_set(selection_data, gdk_atom_intern (GCHEMPAINT_ATOM_NAME, FALSE), 8, (const guchar*) ClipboardData, size);
 		break;
-	case 1:
-	case 2: {
+	case GCP_CLIPBOARD_SVG:
+	case GCP_CLIPBOARD_SVG_XML: {
 		Document *Doc = new Document (NULL, true);
 		View *pView = Doc->GetView ();
 		pView->CreateNewWidget (); // force canvas creation
@@ -128,7 +134,10 @@ static void on_get_data (GtkClipboard *clipboard, GtkSelectionData *selection_da
 		delete Doc;
 		break;
 	}
-	case 3: {
+	case GCP_CLIPBOARD_EPS: {
+		break;
+	}
+	case GCP_CLIPBOARD_PNG: {
 		Document *Doc = new Document (NULL, true);
 		View *pView = Doc->GetView ();
 		gsize size;
@@ -141,7 +150,7 @@ static void on_get_data (GtkClipboard *clipboard, GtkSelectionData *selection_da
 		delete Doc;
 		break;
 	}
-	case 4: {
+	case GCP_CLIPBOARD_JPEG: {
 		Document *Doc = new Document (NULL, true);
 		View *pView = Doc->GetView ();
 		gsize size;
@@ -154,7 +163,7 @@ static void on_get_data (GtkClipboard *clipboard, GtkSelectionData *selection_da
 		delete Doc;
 		break;
 	}
-	case 5: {
+	case GCP_CLIPBOARD_BMP: {
 		Document *Doc = new Document (NULL, true);
 		View *pView = Doc->GetView ();
 		gsize size;
@@ -177,7 +186,7 @@ static void on_get_data (GtkClipboard *clipboard, GtkSelectionData *selection_da
 		App->ActivateWindowsActionWidget ("/MainMenu/EditMenu/Paste", true);
 }
 
-void on_clear_data (GtkClipboard *clipboard, Application *App)
+void on_clear_data (GtkClipboard *clipboard, Object *obj)
 {
 	if (ClipboardData) {
 		xmlFree (ClipboardData);
@@ -186,12 +195,20 @@ void on_clear_data (GtkClipboard *clipboard, Application *App)
 	g_free (ClipboardTextData);
 	ClipboardTextData = NULL;
 	cleared =true;
-	gtk_clipboard_request_contents (clipboard, gdk_atom_intern ("TARGETS", FALSE),  (GtkClipboardReceivedFunc) on_receive_targets, App);
+	Application *app = dynamic_cast <Application *> (obj);
+	if (!app) {
+		// the object might be a tool
+		Tool *tool = dynamic_cast <Tool *> (obj);
+		if (tool)
+			app = tool->GetApplication ();
+	}
+	if (app)
+		gtk_clipboard_request_contents (clipboard, gdk_atom_intern ("TARGETS", FALSE),  (GtkClipboardReceivedFunc) on_receive_targets, app);
 }
 
-bool WidgetData::IsSelected (Object *obj)
+bool WidgetData::IsSelected (Object const *obj) const
 {
-	std::list<Object*>::iterator i, end = SelectedObjects.end ();
+	std::list<Object*>::const_iterator i, end = SelectedObjects.end ();
 	Object* pGroup = obj->GetGroup ();
 	for (i = SelectedObjects.begin (); i != end; i++)
 		if ((*i == obj) || (*i == pGroup))
@@ -199,7 +216,7 @@ bool WidgetData::IsSelected (Object *obj)
 	return false;
 }
 
-void WidgetData::Unselect (Object* obj)
+void WidgetData::Unselect (Object *obj)
 {
 	SelectedObjects.remove (obj);
 	obj->SetSelected (Canvas, SelStateUnselected);
@@ -217,7 +234,7 @@ void WidgetData::UnselectAll ()
 	}
 }
 
-void WidgetData::SetSelected (Object* obj)
+void WidgetData::SetSelected (Object *obj)
 {
 	if (!IsSelected (obj)) {
 		SelectedObjects.push_front (obj);
@@ -301,27 +318,30 @@ void WidgetData::Copy (GtkClipboard* clipboard)
 	gtk_clipboard_request_contents (clipboard, gdk_atom_intern ("TARGETS", FALSE),  (GtkClipboardReceivedFunc) on_receive_targets, App);
 }
 
-void WidgetData::GetObjectBounds (Object* obj, ArtDRect &rect)
+void WidgetData::GetObjectBounds (Object const *obj, ArtDRect &rect) const
 {
-	Object* pObject;
-	GnomeCanvasGroup* group;
+	Object const *pObject;
+	GnomeCanvasGroup const *group;
+	std::map<gcu::Object const *, GnomeCanvasGroup*>::const_iterator g;
 	double x1, y1, x2,  y2;
-	if ((group = Items[obj])) {
+	if ((g = Items.find (obj)) != Items.end ()) {
+		group = (*g).second;
 		gnome_canvas_item_get_bounds (GNOME_CANVAS_ITEM (group), &x1, &y1, &x2, &y2);
-		if (rect.x0 < -9.) {
-			rect.x0 = x1;
-			rect.y0 = y1;
-			rect.x1 = x2;
-			rect.y1 = y2;
-		} else {
-			if (rect.x0 > x1) rect.x0 = x1;
-			if (rect.y0 > y1) rect.y0 = y1;
-			if (rect.x1 < x2) rect.x1 = x2;
-			if (rect.y1 < y2) rect.y1 = y2;
+		if (x2 > 0.) {
+			if (!go_finite (rect.x0)) {
+				rect.x0 = x1;
+				rect.y0 = y1;
+				rect.x1 = x2;
+				rect.y1 = y2;
+			} else {
+				if (rect.x0 > x1) rect.x0 = x1;
+				if (rect.y0 > y1) rect.y0 = y1;
+				if (rect.x1 < x2) rect.x1 = x2;
+				if (rect.y1 < y2) rect.y1 = y2;
+			}
 		}
-	} else
-		Items.erase (obj);
-	std::map<std::string, Object*>::iterator i;
+	}
+	std::map<std::string, Object*>::const_iterator i;
 	pObject = obj->GetFirstChild (i);
 	while (pObject) {
 		GetObjectBounds (pObject, rect);
@@ -329,23 +349,27 @@ void WidgetData::GetObjectBounds (Object* obj, ArtDRect &rect)
 	}
 }
 
-void WidgetData::GetSelectionBounds (ArtDRect &rect)
+void WidgetData::GetSelectionBounds (ArtDRect &rect) const
 {
-	std::list<Object*>::iterator i, end = SelectedObjects.end ();
-	rect.x0 = -10.;
+	std::list<Object*>::const_iterator i, end = SelectedObjects.end ();
+	rect.x0 = go_nan;
 	for (i = SelectedObjects.begin (); i != end; i++)
 		GetObjectBounds (*i, rect);
+	if (!go_finite (rect.x0))
+		rect.x0 = rect.y0 = rect.x1 = rect.y1 = 0.;
 }
 
-void WidgetData::GetObjectBounds (Object* obj, ArtDRect *rect)
+void WidgetData::GetObjectBounds (Object const *obj, ArtDRect *rect) const
 {
-	rect->x0 = -10.;
+	rect->x0 = go_nan;
 	GetObjectBounds (obj, *rect);
+	if (!go_finite (rect->x0))
+		rect->x0 = rect->y0 = rect->x1 = rect->y1 = 0.;
 }
 
 void WidgetData::SelectAll ()
 {
-	std::map<Object*, GnomeCanvasGroup*>::iterator i, end = Items.end ();
+	std::map<Object const*, GnomeCanvasGroup*>::iterator i, end = Items.end ();
 	Object *pObject;
 	for (i = Items.begin (); i != end; i++) {
 		pObject = (*i).first->GetGroup ();
@@ -354,7 +378,7 @@ void WidgetData::SelectAll ()
 				SetSelected (pObject);
 		}
 		else if (!IsSelected ((*i).first))
-			SetSelected ((*i).first);
+			SetSelected (const_cast <Object *> ((*i).first));
 	}
 }
 
